'''Jan 2016 Aaron Tuor
Tool to process Movielens 100k Metadata
'''
import argparse
import numpy
import scipy.sparse as sps
import os
import scipy.io

slash = '/'
if os.name == 'nt':
    slash = '\\'  # so this works in Windows
parser = argparse.ArgumentParser()
parser.add_argument('datapath', type=str)
parser.add_argument('outpath', type=str, default='')
args = parser.parse_args()
if not args.datapath.endswith(slash):
    args.datapath += slash
infile = open(args.datapath + 'u.occupation', 'r')
monthlist = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec']
monthmap = {k: v for k, v in zip(monthlist, range(12))}
infile.close()
item_features = []
with open(args.datapath + 'u2.item') as infile:
    first = False
    lines = infile.readlines()
    for line in lines:
        item_feature_list = []
        line = line.split('|')
        # month released==================================
        date = line[2].split('-')
        months = [0.0]*12
        months[monthmap[date[1]]] = 1.0
        item_feature_list.extend(months)
        # year released==================================
        item_feature_list.append(float(int(date[2]) - 1930)/(2016.0-1930.0))
        print(line)
        genres = line[5:len(line)]
        for i in range(len(genres)):
            genres[i] = float(genres[i])
        item_feature_list.extend(genres)
        item_features.append(item_feature_list)
        print(len(item_features))
    item_feature_matrix = numpy.array(item_features)
    item_feature_matrix = sps.csr_matrix(item_feature_matrix)
    print(item_feature_matrix.shape)
    scipy.io.savemat(args.outpath, {'data': item_feature_matrix})


'''February 2016 Aaron Tuor
Tool to make 5/5/5/5/80, coldDev, coldTest, dev, test, train datasplit for movielens 100k with imdb plot descriptions.
'''
import argparse
import os
import scipy.sparse as sps
import scipy.io
import loader
import random
import numpy as np

slash = '/'
if os.name == 'nt':
    slash = '\\'  # so this works in Windows

# Handle arguments
parser = argparse.ArgumentParser()
parser.add_argument('readpath', type=str,
                    help='Path to folder where data to be split is located.')
parser.add_argument('writepath', type=str,
                    help='Path to folder where split data is to be stored: Folder must contain, train, dev and'
                         'test directories.')
parser.add_argument('ratings',
                    help='Filename of ratings file datafiles to use in the split.')
args = parser.parse_args()

# slash ambivalent
if not args.readpath.endswith(slash):
    args.readpath += slash
if not args.writepath.endswith(slash):
    args.writepath += slash

loader.makedirs(args.writepath)

# read utility matrix and independently shuffle rows and columns
ratings = loader.import_data(args.readpath + args.ratings)
num_users = ratings.shape[0]
num_items = ratings.shape[1]
num_ratings = ratings.getnnz()
order = range(num_ratings)
print(len(order))
random.shuffle(order)
split_size = int(round(num_ratings*.05))

# save shuffled orders
scipy.io.savemat(args.writepath+'order.mat', {'data': np.array(order)})



#========================================================================================
#========================================================================================
def toOnehot(X, dim):
    #empty one-hot matrix
    hotmatrix = np.zeros((X.shape[0], dim))
    #fill indice positions
    hotmatrix[np.arange(X.shape[0]), X] = 1
    hotmatrix = sps.csr_matrix(hotmatrix)
    return hotmatrix

# split
[i, j, v] = sps.find(ratings)
i = i[order]
j = j[order]
v = v[order]
# find dev set
print(split_size)
dev_users = i[range(split_size)]
dev_items = j[range(split_size)]
# scipy.io.savemat(args.writepath+'dev/features_user.mat', {'data': dev_users})
# scipy.io.savemat(args.writepath+'dev/features_item.mat', {'data': dev_items})
# scipy.io.savemat(args.writepath+'dev/labels_values.mat', {'data': v[range(split_size)]})
scipy.io.savemat(args.writepath+'dev/features_user.mat', {'data': toOnehot(dev_users, num_users)})
scipy.io.savemat(args.writepath+'dev/features_item.mat', {'data': toOnehot(dev_items, num_items)})
scipy.io.savemat(args.writepath+'dev/labels_values.mat', {'data': v[range(split_size)]})

# find test set
test_start = split_size
test_end = 2*split_size
test_users = i[test_start:test_end]
test_items = j[test_start:test_end]
scipy.io.savemat(args.writepath+'test/features_user.mat', {'data': toOnehot(test_users, num_users)})
scipy.io.savemat(args.writepath+'test/features_item.mat', {'data': toOnehot(test_items, num_items)})
scipy.io.savemat(args.writepath+'test/labels_values.mat', {'data': v[test_start:test_end]})

# find train set
train_start = test_end
train_users = i[range(train_start, 80000)]
train_items = j[range(train_start, 80000)]
scipy.io.savemat(args.writepath+'train/features_user.mat', {'data': toOnehot(train_users, num_users)})
scipy.io.savemat(args.writepath+'train/features_item.mat', {'data': toOnehot(train_items, num_items)})
scipy.io.savemat(args.writepath+'train/labels_values.mat', {'data': v[train_start:80000]})

numnoncoldratings = v[range(split_size)].shape[0] + v[range(test_start, test_end)].shape[0] + v[range(train_start, num_ratings)].shape[0]

assert(numnoncoldratings == num_ratings)




import loader
import scipy.io as sio

a = loader.read_data_sets('/home/aarontuor/data/ml100k')
a.show()
devhots = loader.toOnehot(a.dev.labels['ratings'] -1, 5)
# trainhots = loader.toOnehot(a.train.labels['ratings'] -1, 5)
testhots = loader.toOnehot(a.test.labels['ratings'] -1, 5)
print(devhots.todense().shape)

loader.smatsave('/home/aarontuor/test.sparse', devhots.todense())


'''February 2016 Aaron Tuor
Tool to make 5/5/5/5/80, coldDev, coldTest, dev, test, train datasplit for movielens 100k with imdb plot descriptions.
'''
import argparse
import os
import scipy.sparse as sps
import scipy.io
import loader
import random
import numpy as np

slash = '/'
if os.name == 'nt':
    slash = '\\'  # so this works in Windows

# Handle arguments
parser = argparse.ArgumentParser()
parser.add_argument('readpath', type=str,
                    help='Path to folder where data to be split is located.')
parser.add_argument('writepath', type=str,
                    help='Path to folder where split data is to be stored: Folder must contain, train, dev and'
                         'test directories.')
parser.add_argument('ratings',
                    help='Filename of ratings file datafiles to use in the split.')
parser.add_argument('-item_list', default=[], nargs='+',
                    help='One or more item datafiles to use in the split.')
parser.add_argument('-user_list', default=[], nargs='+',
                    help='List of user datafiles to use in the split.')
args = parser.parse_args()

# slash ambivalent
if not args.readpath.endswith(slash):
    args.readpath += slash
if not args.writepath.endswith(slash):
    args.writepath += slash

loader.makedirs(args.writepath, cold=True)

# read utility matrix and independently shuffle rows and columns
ratings = loader.import_data(args.readpath + args.ratings)
num_users = ratings.shape[0]
num_items = ratings.shape[1]
num_ratings = ratings.getnnz()
item_order = range(num_items)
user_order = range(num_users)
random.shuffle(item_order)
random.shuffle(user_order)

# save shuffled orders
scipy.io.savemat(args.writepath+'item_order.mat', {'data': np.array(item_order)})
scipy.io.savemat(args.writepath+'user_order.mat', {'data': np.array(user_order)})
ratings = ratings[:, item_order]
ratings = ratings[user_order, :]
split_size = num_ratings*0.05

# find item test_cold set
i = num_items
num_cold_test_items = 0
while num_cold_test_items < split_size:
    i = i - 1
    num_cold_test_items = num_cold_test_items + ratings[:, i].getnnz()
item_start_test_cold = i

# find item dev_cold set
num_cold_dev_items = 0
item_end_dev_cold = i
while num_cold_dev_items < split_size:
    i = i - 1
    num_cold_dev_items = num_cold_dev_items + ratings[:, i].getnnz()
item_start_dev_cold = i

#======================================================================
# find user test_cold set
i = num_users
num_cold_test = 0
while num_cold_test < split_size:
    i = i - 1
    num_cold_test = num_cold_test + ratings[i, :].getnnz()
user_start_test_cold = i
# ------------------------------------------------------------

# find user dev_cold set
num_cold_dev = 0
end_dev_cold = i
while num_cold_dev < split_size:
    i = i - 1
    num_cold_dev = num_cold_dev + ratings[i, :].getnnz()
user_start_dev_cold = i
# --------------------------------------------------------------

#========================================================================================
#========================================================================================
def toOnehot(X, dim):
    #empty one-hot matrix
    hotmatrix = np.zeros((X.shape[0], dim))
    #fill indice positions
    hotmatrix[np.arange(X.shape[0]), X] = 1
    hotmatrix = sps.csr_matrix(hotmatrix)
    return hotmatrix
#set target idx to 1
#save cold sets
item_test_cold = ratings[0:user_start_dev_cold, item_start_test_cold:num_items]
itemtestcoldusers, itemtestcolditems, itemtestcoldvalues = sps.find(item_test_cold)

scipy.io.savemat(args.writepath+'test_cold_item/features_user.mat', {'data': toOnehot(itemtestcoldusers, item_test_cold.shape[0])})
scipy.io.savemat(args.writepath+'test_cold_item/features_item.mat', {'data': toOnehot(itemtestcolditems, item_test_cold.shape[1])})
scipy.io.savemat(args.writepath+'test_cold_item/labels_values.mat', {'data': itemtestcoldvalues})

item_dev_cold = ratings[0:user_start_dev_cold, item_start_dev_cold:item_end_dev_cold]
itemdevcoldusers, itemdevcolditems, itemdevcoldvalues = sps.find(item_dev_cold)
scipy.io.savemat(args.writepath+'dev_cold_item/features_user.mat', {'data': toOnehot(itemdevcoldusers, item_dev_cold.shape[0])})
scipy.io.savemat(args.writepath+'dev_cold_item/features_item.mat', {'data': toOnehot(itemdevcolditems, item_dev_cold.shape[1])})
scipy.io.savemat(args.writepath+'dev_cold_item/labels_values.mat', {'data': itemdevcoldvalues})

user_test_cold = ratings[user_start_test_cold:num_users, 0:item_start_dev_cold]
usertestcoldusers, usertestcolditems, usertestcoldvalues = sps.find(user_test_cold)
scipy.io.savemat(args.writepath+'test_cold_user/features_user.mat', {'data': toOnehot(usertestcoldusers, user_test_cold.shape[0])})
scipy.io.savemat(args.writepath+'test_cold_user/features_item.mat', {'data': toOnehot(usertestcolditems, user_test_cold.shape[1])})
scipy.io.savemat(args.writepath+'test_cold_user/labels_values.mat', {'data': usertestcoldvalues})

user_dev_cold = ratings[user_start_dev_cold:end_dev_cold, 0:item_start_dev_cold]
userdevcoldusers, userdevcolditems, userdevcoldvalues = sps.find(user_dev_cold)
scipy.io.savemat(args.writepath+'dev_cold_user/features_user.mat', {'data': toOnehot(userdevcoldusers, user_dev_cold.shape[0])})
scipy.io.savemat(args.writepath+'dev_cold_user/features_item.mat', {'data': toOnehot(userdevcolditems, user_dev_cold.shape[1])})
scipy.io.savemat(args.writepath+'dev_cold_user/labels_values.mat', {'data': userdevcoldvalues})

both_cold = ratings[user_start_dev_cold:num_users, item_start_dev_cold: num_items]
bothcoldusers, bothcolditems, bothcoldvalues = sps.find(both_cold)
scipy.io.savemat(args.writepath+'both_cold/features_user.mat', {'data': toOnehot(bothcoldusers, both_cold.shape[0])})
scipy.io.savemat(args.writepath+'both_cold/features_item.mat', {'data': toOnehot(bothcolditems, both_cold.shape[1])})
scipy.io.savemat(args.writepath+'both_cold/labels_values.mat', {'data': bothcoldvalues})


# split
[i, j, v] = sps.find(ratings[0:user_start_dev_cold, 0:item_start_dev_cold])
num_ratings = ratings[0:user_start_dev_cold, 0:item_start_dev_cold].getnnz()
order = range(i.shape[0])
random.shuffle(order)
i = i[order]
j = j[order]
v = v[order]
# find dev set
split_size = int(round(split_size))
dev_users = i[range(split_size)]
dev_items = j[range(split_size)]
scipy.io.savemat(args.writepath+'dev/features_user.mat', {'data': toOnehot(dev_users, num_users)})
scipy.io.savemat(args.writepath+'dev/features_item.mat', {'data': toOnehot(dev_items, num_items)})
scipy.io.savemat(args.writepath+'dev/labels_values.mat', {'data': v[range(split_size)]})

# find test set
test_start = split_size
test_end = 2*split_size
test_users = i[test_start:test_end]
test_items = j[test_start:test_end]
scipy.io.savemat(args.writepath+'test/features_user.mat', {'data': toOnehot(test_users, num_users)})
scipy.io.savemat(args.writepath+'test/features_item.mat', {'data': toOnehot(test_items, num_items)})
scipy.io.savemat(args.writepath+'test/labels_values.mat', {'data': v[test_start:test_end]})

# find train set
train_start = test_end
train_users = i[range(train_start, num_ratings)]
train_items = j[range(train_start, num_ratings)]
scipy.io.savemat(args.writepath+'train/features_user.mat', {'data': toOnehot(train_users, num_users)})
scipy.io.savemat(args.writepath+'train/features_item.mat', {'data': toOnehot(train_items, num_items)})
scipy.io.savemat(args.writepath+'train/labels_values.mat', {'data': v[train_start:num_ratings]})


 numnoncoldratings = v[range(split_size)].shape[0] + v[range(test_start, test_end)].shape[0] + v[range(train_start, num_non_cold)].shape[0]
 assert(numcoldratings + numnoncoldratings == num_ratings)

# split extra user and item features
print('gothere')
for u in args.user_list:
    namestub = os.path.splitext(os.path.basename(u))[0]
    matrix = loader.import_data(args.readpath + u)
    scipy.io.savemat(args.writepath+'train/features_'+namestub+'.mat',
                     {'data': matrix[train_users.flatten()]})
    scipy.io.savemat(args.writepath+'dev/features_'+namestub+'.mat',
                     {'data': matrix[dev_users.flatten()]})
    scipy.io.savemat(args.writepath+'test/features_'+namestub+'.mat',
                     {'data': matrix[test_users.flatten()]})
    scipy.io.savemat(args.writepath+'test_cold_item/features_'+namestub+'.mat',
                     {'data': matrix[itemtestcoldusers.flatten()]})
    scipy.io.savemat(args.writepath+'test_cold_user/features_'+namestub+'.mat',
                     {'data': matrix[usertestcoldusers.flatten()]})
    scipy.io.savemat(args.writepath+'dev_cold_item/features_'+namestub+'.mat',
                     {'data': matrix[itemdevcoldusers.flatten()]})
    scipy.io.savemat(args.writepath+'dev_cold_user/features_'+namestub+'.mat',
                     {'data': matrix[userdevcoldusers.flatten()]})
    scipy.io.savemat(args.writepath+'both_cold/features_'+namestub+'.mat',
                     {'data': matrix[bothcoldusers.flatten()]})

for i in args.item_list:
    print(np.max(train_items))
    namestub = os.path.splitext(os.path.basename(i))[0]
    matrix = loader.import_data(args.readpath + i)
    scipy.io.savemat(args.writepath+'train/features_'+namestub+'.mat',
                     {'data': matrix[train_items.flatten()]})
    scipy.io.savemat(args.writepath+'dev/features_'+namestub+'.mat',
                     {'data': matrix[dev_items.flatten()]})
    scipy.io.savemat(args.writepath+'test/features_'+namestub+'.mat',
                     {'data': matrix[test_items.flatten()]})
    scipy.io.savemat(args.writepath+'test_cold_item/features_'+namestub+'.mat',
                     {'data': matrix[itemtestcolditems.flatten()]})
    scipy.io.savemat(args.writepath+'test_cold_user/features_'+namestub+'.mat',
                     {'data': matrix[usertestcolditems.flatten()]})
    scipy.io.savemat(args.writepath+'dev_cold_item/features_'+namestub+'.mat',
                     {'data': matrix[itemdevcolditems.flatten()]})
    scipy.io.savemat(args.writepath+'dev_cold_user/features_'+namestub+'.mat',
                     {'data': matrix[userdevcolditems.flatten()]})
    scipy.io.savemat(args.writepath+'both_cold/features_'+namestub+'.mat',
                     {'data': matrix[bothcolditems.flatten()]})




import os
import argparse

parser = argparse.ArgumentParser()
parser.add_argument('folderpath', type=str)
args = parser.parse_args()
folder = args.folderpath
if folder.endswith('/'):
    folder = folder[0:len(folder)-1]
os.system('mkdir ' + folder)
os.system('mkdir ' + folder + '/dev')
os.system('mkdir ' + folder + '/train')
os.system('mkdir ' + folder + '/test')




'''February 2016 Aaron Tuor
Make vectors of indices and values from sparse matrix format text files
'''
import argparse
import sys
import re
import math
import os
import numpy as np
import scipy.sparse as sps
import scipy.io
slash = '/'
if os.name == 'nt':
    slash = '\\'  # so this works in Windows
# Handle arguments
parser = argparse.ArgumentParser()
parser.add_argument('datapath', type=str,
                    help='Path to sparse matrix text file')
parser.add_argument('filename', type=str,
                    help='Name of sparse matrix text file')
args = parser.parse_args()
# slash ambivalent
if not args.datapath.endswith(slash):
    args.datapath += slash

sparse = np.loadtxt(args.datapath + args.filename)
rows = sparse[:, 0]
cols = sparse[:, 1]
vals = sparse[:, 2]

print(rows.shape)
print(cols.shape)
print(vals.shape)
scipy.io.savemat(args.datapath + 'user', {'data': rows})
scipy.io.savemat(args.datapath + 'item', {'data': cols})
scipy.io.savemat(args.datapath + 'values', {'data': vals})



'''February 2016 Aaron Tuor
Tool to make numpy .mat term_doc files from descriptions and dictionary files.
'''
import argparse
import os
import numpy as np
import scipy.sparse as sps
import scipy.io

slash = '/'
if os.name == 'nt':
    slash = '\\'  # so this works in Windows

# Handle arguments
parser = argparse.ArgumentParser()
parser.add_argument('datapath', type=str,
                    help='Path to folder where dictionary and descriptions are located, and created document '
                                'term matrix will be saved.')
parser.add_argument('dictionary', type=str,
                    help='Name of the file containing line separated words in vocabulary.')
parser.add_argument('descriptions', type=str,
                    help='Name of the file containing line separated text descriptions.')
parser.add_argument('doc_term_file', type=str,
                    help='Name of the file to save the created sparse document term matrix.')
args = parser.parse_args()

# slash ambivalent
if not args.datapath.endswith(slash):
    args.datapath += slash

# make hashmap of words to Integers
dictionaryFile = open(args.datapath + args.dictionary, 'r')
lexicon = dictionaryFile.read().strip().split('\n')
wordmap = {k: v for k, v in zip(lexicon, range(len(lexicon)))}
# open read and write files
outfile = open(args.datapath + args.doc_term_file, "w")
descriptionFile = open(args.datapath + args.descriptions, 'r')
# go through each line in file
docterm = []
for line in descriptionFile:
    countArray = [0] * len(wordmap)  # counts for words in each product description
    lineArray = line.split()
    for word in lineArray:
        if word in wordmap:
            countArray[wordmap[word]] += 1
    docterm.append(countArray)
outfile.close()
doc_term_matrix = sps.csr_matrix(np.array(docterm))
print(doc_term_matrix.shape)
scipy.io.savemat(args.datapath+args.doc_term_file, {'data': doc_term_matrix})


'''Jan 2016 Aaron Tuor
Tool to process Movielens 100k Metadata
'''
import argparse
import numpy
import scipy.sparse as sps
import os
import scipy.io

slash = '/'
if os.name == 'nt':
    slash = '\\'  # so this works in Windows
parser = argparse.ArgumentParser()
parser.add_argument('datapath', type=str)
parser.add_argument('outpath', type=str, default='')
args = parser.parse_args()
if not args.datapath.endswith(slash):
    args.datapath += slash
infile = open(args.datapath + 'u.occupation', 'r')
occlist = infile.read()
occlist = occlist.split('\n')
occmap = {k: v for k, v in zip(occlist, range(len(occlist)))}
monthlist = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec']
monthmap = {k: v for k, v in zip(monthlist, range(1, 13))}
print(monthmap)
infile.close()
user_features = []
with open(args.datapath + 'u.user') as infile:
    first = False
    lines = infile.readlines()
    for line in lines:
        user_feature_list = []
        line = line.split('|')
        # age==================================
        age = float(line[1]) / 100
        user_feature_list.append(age)
        # sex==================================
        if line[2] == 'M':
            user_feature_list.append(1.0)
            user_feature_list.append(0.0)
        else:
            user_feature_list.append(0.0)
            user_feature_list.append(1.0)
        # occupation==================================
        occupation = [0.0] * len(occlist)
        occupation[occmap[line[3]]] = 1.0
        user_feature_list.extend(occupation)
        # first zip code digit: large geographical areas
        zip1 = [0.0] * 10
        if not line[4].isdigit():
            line[4] = '00000'
        zip1[int(line[4][0])] = 1.0
        user_feature_list.extend(zip1)
        # next two digits: regional centers
        zip2 = [0.0] * 100
        zip2[int(line[4][1:3])] = 1.0
        user_feature_list.extend(zip2)
        user_features.append(user_feature_list)
        user_feature_matrix = numpy.array(user_features)
        user_feature_matrix = sps.csr_matrix(user_feature_matrix)
        print(user_feature_matrix.shape)
    scipy.io.savemat(args.outpath, {'data': user_feature_matrix})



#!/usr/bin/env python
from ANT.core import loader
import argparse

# Handle arguments
def return_parser():
    parser = argparse.ArgumentParser()
    parser.add_argument('datadirectory', type=str,
                        help='Path to folder where data to be loaded and displayed is stored.')

    parser.add_argument('-hashlist', nargs='+',
                        help='Path to folder where data to be loaded and displayed is stored.')
    subfolder_group = parser.add_mutually_exclusive_group()
    subfolder_group.add_argument('-cold', action="store_true",
                                 help="Extra loading and testing for cold datasets")
    subfolder_group.add_argument('-subfolders', default=('test', 'dev', 'train'), nargs='+',
                        help='List of subfolders to load and display.')
    return parser


if __name__ == '__main__':
    args = return_parser().parse_args()
    if args.cold:
        args.subfolders = ['train', 'dev', 'test', 'dev_cold_item', 'dev_cold_user',
                           'test_cold_item', 'test_cold_user', 'both_cold']
    loader.read_data_sets(args.datadirectory, args.subfolders, args.hashlist).show()


'''February 2016 Aaron Tuor
Tool to make 5/5/5/5/80, coldDev, coldTest, dev, test, train datasplit for movielens 100k with imdb plot descriptions.
'''
import argparse
import os
import scipy.sparse as sps
import scipy.io
import loader
import random
import numpy as np

slash = '/'
if os.name == 'nt':
    slash = '\\'  # so this works in Windows

# Handle arguments
parser = argparse.ArgumentParser()
parser.add_argument('readpath', type=str,
                    help='Path to folder where data to be split is located.')
parser.add_argument('writepath', type=str,
                    help='Path to folder where split data is to be stored: Folder must contain, train, dev and'
                         'test directories.')
parser.add_argument('-item_list', default=[], nargs='+',
                    help='One or more item datafiles to use in the split.')
parser.add_argument('ratings',
                    help='Filename of ratings file datafiles to use in the split.')
parser.add_argument('-user_list', default=[], nargs='+',
                    help='List of user datafiles to use in the split.')
args = parser.parse_args()

# slash ambivalent
if not args.readpath.endswith(slash):
    args.readpath += slash
if not args.writepath.endswith(slash):
    args.writepath += slash

loader.makedirs(args.writepath)

# read utility matrix and independently shuffle rows and columns
ratings = loader.import_data(args.readpath + args.ratings)
num_users = ratings.shape[0]
num_items = ratings.shape[1]
num_ratings = ratings.getnnz()
item_order = range(num_items)
user_order = range(num_users)
random.shuffle(item_order)
random.shuffle(user_order)

# save shuffled orders
scipy.io.savemat(args.writepath+'item_order.mat', {'data': np.array(item_order)})
scipy.io.savemat(args.writepath+'user_order.mat', {'data': np.array(user_order)})
ratings = ratings[:, item_order]
ratings = ratings[user_order, :]
split_size = num_ratings*0.05

# find item test_cold set
i = num_items
num_cold_test_items = 0
while num_cold_test_items < split_size:
    i = i - 1
    num_cold_test_items = num_cold_test_items + ratings[:, i].getnnz()
item_start_test_cold = i

# find item dev_cold set
num_cold_dev_items = 0
item_end_dev_cold = i
while num_cold_dev_items < split_size:
    i = i - 1
    num_cold_dev_items = num_cold_dev_items + ratings[:, i].getnnz()
item_start_dev_cold = i

#======================================================================
# find user test_cold set
i = num_users
num_cold_test = 0
while num_cold_test < split_size:
    i = i - 1
    num_cold_test = num_cold_test + ratings[i, :].getnnz()
user_start_test_cold = i
# ------------------------------------------------------------

# find user dev_cold set
num_cold_dev = 0
end_dev_cold = i
while num_cold_dev < split_size:
    i = i - 1
    num_cold_dev = num_cold_dev + ratings[i, :].getnnz()
user_start_dev_cold = i
# --------------------------------------------------------------

#========================================================================================
#========================================================================================

#save cold sets
item_test_cold = ratings[0:user_start_dev_cold, item_start_test_cold:num_items]
scipy.io.savemat(args.writepath+'test/features_item_cold.mat', {'data': item_test_cold})
item_dev_cold = ratings[0:user_start_dev_cold, item_start_dev_cold:item_end_dev_cold]
scipy.io.savemat(args.writepath+'dev/features_item_cold.mat', {'data': item_dev_cold})
user_test_cold = ratings[user_start_test_cold:num_users, 0:item_start_dev_cold]
scipy.io.savemat(args.writepath+'test/features_user_cold.mat', {'data': user_test_cold})
user_dev_cold = ratings[user_start_dev_cold:end_dev_cold, 0:item_start_dev_cold]
scipy.io.savemat(args.writepath+'test/features_item_cold.mat', {'data': user_test_cold})
both_cold = ratings[user_start_dev_cold:num_users, item_start_dev_cold: num_items]
scipy.io.savemat(args.writepath+'test/features_both_cold.mat', {'data': both_cold})


# find non cold sets
non_coldratings = ratings[0:user_start_dev_cold, 0:item_start_dev_cold]
[i, j, v] = sps.find(non_coldratings)
num_non_cold = i.shape[0]
numcoldratings = item_test_cold.getnnz() + item_dev_cold.getnnz() + user_test_cold.getnnz() + user_dev_cold.getnnz() + both_cold.getnnz()
split_size = int(round(split_size))
print(split_size)
# find dev set
dev_users = i[range(split_size)]
dev_items = j[range(split_size)]
scipy.io.savemat(args.writepath+'dev/features_user.mat', {'data': i[range(split_size)]})
scipy.io.savemat(args.writepath+'dev/features_item.mat', {'data': j[range(split_size)]})
scipy.io.savemat(args.writepath+'dev/labels_values.mat', {'data': v[range(split_size)]})

# find test set
test_start = split_size
test_end = 2*split_size
test_users = i[test_start:test_end]
test_items = j[test_start:test_end]
scipy.io.savemat(args.writepath+'test/features_user.mat', {'data': i[test_start:test_end]})
scipy.io.savemat(args.writepath+'test/features_item.mat', {'data': j[range(test_start, test_end)]})
scipy.io.savemat(args.writepath+'test/labels_values.mat', {'data': v[range(test_start, test_end)]})

# find train set
train_start = test_end
train_users = i[range(train_start, num_non_cold)]
train_items = j[range(train_start, num_non_cold)]
scipy.io.savemat(args.writepath+'train/features_user.mat', {'data': i[range(train_start, num_non_cold)]})
scipy.io.savemat(args.writepath+'train/features_item.mat', {'data': j[range(train_start, num_non_cold)]})
scipy.io.savemat(args.writepath+'train/labels_values.mat', {'data': v[range(train_start, num_non_cold)]})

numnoncoldratings = v[range(split_size)].shape[0] + v[range(test_start, test_end)].shape[0] + v[range(train_start, num_non_cold)].shape[0]
assert(numcoldratings + numnoncoldratings)

# split extra user and item features
for u in args.user_list:
    namestub = os.path.splitext(os.path.basename(u))[0]
    matrix = loader.import_data(args.readpath + u)
    print(matrix.shape)
    print(dev_users.flatten().shape)
    scipy.io.savemat(args.writepath+'train/features_'+namestub+'.mat',
                     {'data': matrix[user_order[0:user_start_dev_cold]]})
    scipy.io.savemat(args.writepath+'dev/features_'+namestub+'.mat',
                     {'data': matrix[user_order[user_start_dev_cold:user_start_test_cold]]})
    scipy.io.savemat(args.writepath+'test/features_'+namestub+'.mat',
                     {'data': matrix[user_order[user_start_test_cold:num_items]]})
    assert(num_users == matrix.shape[0])
    assert(num_users == (matrix[user_order[0:user_start_dev_cold]].shape[0] +
                         matrix[user_order[user_start_dev_cold:user_start_test_cold]].shape[0] +
                         matrix[user_order[user_start_test_cold:num_items]].shape[0]))

print(args.item_list)
for i in args.item_list:
    namestub = os.path.splitext(os.path.basename(i))[0]
    matrix = loader.import_data(args.readpath + i)
    scipy.io.savemat(args.writepath+'train/features_'+namestub+'.mat',
                     {'data': matrix[item_order[0:item_start_dev_cold]]})
    scipy.io.savemat(args.writepath+'dev/features_'+namestub+'.mat',
                     {'data': matrix[item_order[item_start_dev_cold:item_start_test_cold]]})
    scipy.io.savemat(args.writepath+'test/features_'+namestub+'.mat',
                     {'data': matrix[item_order[item_start_test_cold:num_users]]})
    assert(num_items == matrix.shape[0])
    assert(num_items == (matrix[item_order[0:item_start_dev_cold]].shape[0] +
                         matrix[item_order[item_start_dev_cold:item_start_test_cold]].shape[0] +
                         matrix[item_order[item_start_test_cold:num_users]].shape[0]))






try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup

setup(name='antk',
      version=0.1,
      description='Automated Neural-graph Toolkit: A Tensorflow wrapper for '
                  'common deep learning tasks and rapid development of new'
                  'models.',
      url='http://aarontuor.xyz',
      author='Aaron Tuor',
      author_email='tuora@students.wwu.edu',
      license='none',
      packages=['ANT/datascripts'],
      scripts=['ANT/datascripts/datatest.py', 'ANT/datascripts/normalize.py',
               'ANT/antdocs.py'],
      zip_safe=False,
      install_requires=['scipy', 'numpy', 'tensorflow'],
      classifiers=[
        'Programming Language :: Python',
        'Operating System :: OS Independent',
        'Intended Audience :: Science/Research',
        'Topic :: Scientific/Engineering'],
    keywords=[
        'Deep Learning',
        'Neural Networks',
        'TensorFlow',
        'Machine Learning',
        'Western Washington University'])


'''
normalize.py  : Text normalization script
authors       : Elliot Starks, Aaron Tuor, Samuel Nguyen
last modified : August, 2015
'''

#!/user/bin/env python

import argparse
import sys
import re

# Handle arguments
arguments = sys.argv
if len(arguments) < 1:
    print 'Usage: <infile>'
    sys.exit();

infile = open(arguments[1], 'r')
namestub = arguments[1].split('.')[0].strip()
extension = arguments[1].split('.')[1].strip()
newfile = namestub + '_norm.' + extension
outfile = open(newfile, 'w')

# Pre-compile regular expressions
regexPunc = re.compile(r"[^A-Za-z0-9']")

# Process each line
for line in infile:
    line = regexPunc.sub(" ", line)#replace punctuation with space excepting apostrophes
    line = line.lower() # make everything lowercase		
    wordlist = line.split(" ")#make an array of words to individually work on words
	
# Process each word; write words delimited by space to file
    for word in wordlist:
	word = word.strip('\'') #yank out leading and trailing quotation marks
	word = word.strip()
	if word != '':
		outfile.write(word)
		outfile.write(' ')
    outfile.write("\n")
outfile.close()




